<?php

namespace App\Http\Controllers;

use App\Events\AppointmentCreatedOrUpdatedEvent;
use App\Http\Requests\StoreAppointmentRequest;
use App\Http\Requests\UpdateAppointmentRequest;
use App\Models\Appointment;
use App\Models\Organization;
use App\Notifications\AppointmentDeletedNotification;
use App\Notifications\AppointmentUpdatedNotification;
use App\Services\AppointmentService;
use App\Services\MessageService;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Request;
use Inertia\Inertia;

class AppointmentController extends Controller
{
    public function __construct()
    {
        $this->middleware('permission:view_appointments|view_all_appointments')->only(['index', 'show']);
        $this->middleware('permission:edit_appointments|edit_all_appointments')->only(['edit', 'update']);
        $this->middleware('permission:publish_appointments|publish_all_appointments')->only(['create', 'store']);
        $this->middleware('permission:publish_appointments|publish_all_appointments')->only(['create', 'store']);
        $this->middleware(['permission:delete_appointments|delete_all_appointments|delete_counselor_appointment'])->only(['destroy']);
    }

    public function index(Organization $organization, Appointment $appointment = null)
    {
        $appointments = Appointment::query()
            ->whereBelongsTo($organization)
            ->when(!Request::input('all') || Request::input('all') != 1 || !auth()->user()->hasPermissionTo('view_all_appointments'), function ($query, $all) {
                $query->belongsToUser();
            })
            ->when(Request::input('all') && Request::input('all') == 1 && auth()->user()->hasPermissionTo('view_all_appointments'), function ($query, $all) {
                $query->where('type', 'internal');
            })
            ->select(['id', 'organization_id', 'conversation_id', 'static_title', 'type', 'location', 'description', 'start', 'end', 'status', 'appointment_type'])
            ->orderBy('start')
            ->with(['users:id,first_name,last_name,email,phone_number', 'conversation:id,uuid,organization_id'])
            ->get();

        $search = Request::only('all');

        $user_conversations = auth()->user()->conversations->where('organization_id', $organization->id);

        return Inertia::render('Appointments/Index', [
            'appointments' => $appointments,
            'appointment' => $appointment,
            'filters' => $search,
            'user_conversations' => $user_conversations,
        ]);
    }

    public function store(StoreAppointmentRequest $request, Organization $organization, AppointmentService $appointmentService, MessageService $messageService)
    {
        $validated = $request->validated();

        $validated['organization_id'] = $organization->id;

        $validated = $appointmentService->setStartEndTime($validated);

        $appointment = Appointment::create($validated);

        if ($appointment->conversation && $appointment->conversation->users->isNotEmpty()) {
            $appointment->users()->attach($appointment->conversation->users);
            $appointment->save();
        } else {
            $appointment->users()->attach(auth()->user());
            $appointment->save();
        }

        if ($validated['type'] == 'internal') {

            $recipient = $appointment->users()->whereHas('roles', function (Builder $query) {
                $query->where('name', 'client');
            })->first();

            $counselor = $appointment->users()->whereHas('roles', function (Builder $query) {
                $query->where('name', 'counselor')
                    ->orWhere('name', 'organization-admin')
                    ->orWhere('name', 'super-admin');
            })->first();

            if ($recipient && $counselor) {
                Notification::route('mail', $recipient->email)
                    ->notify((new \App\Notifications\AppointmentCreatedNotification($appointment, $organization, $recipient, $counselor))->locale(app()->getLocale() ?? 'en'));
            }

        }

        if ($validated['broadcast'] && $validated['type'] == 'internal') {
            broadcast(new AppointmentCreatedOrUpdatedEvent($appointment->conversation, $appointment));
            $message = __('Appointment created') . ': ' . Carbon::createFromFormat('Y-m-d H:i:s', $appointment->start)->format('d.m.Y (H:i') . ' - ' . Carbon::createFromFormat('Y-m-d H:i:s', $appointment->end)->format('H:i') . ')';
            $messageService->createMessage($message, $appointment->conversation, $organization, 'appointment');

            return redirect()->back();
        }

        return redirect()->back()->with('success', 'Appointment created.');
    }

    public function update(Organization $organization, Appointment $appointment, UpdateAppointmentRequest $request, AppointmentService $appointmentService)
    {
        $validated = $request->validated();

        $validated = $appointmentService->setStartEndTime($validated);

        $notifyClient = $validated['start'] != $appointment->start || $validated['end'] != $appointment->end;

        $appointment->update($validated);

        if ($appointment->conversation && $appointment->conversation->users->isNotEmpty() && $appointment->users != $appointment->conversation->users) {
            $appointment->users()->sync($appointment->conversation->users);
            $appointment->save();
        }

        if ($notifyClient && $appointment->clients->isNotEmpty()) {
            $appointment->clients->each(function ($client) use ($appointment) {
                $client->notify((new AppointmentUpdatedNotification($appointment, $appointment->organization, $client, $appointment->counselors()->first()))->locale(app()->getLocale() ?? 'en'));
            });
        }


        if ($validated['broadcast'] && $validated['type'] == 'internal') {
            broadcast(new AppointmentCreatedOrUpdatedEvent($appointment->conversation, $appointment));
        }
        //return Redirect::route('organizations.appointments.index', ['organization' => auth()->user()->currentOrganization])->with('success', 'Appointment updated.');

        return redirect()->back()->with('success', 'Appointment updated.');
    }

    public function destroy(Organization $organization, Appointment $appointment)
    {
        $appointment_details = $appointment->toArray();

        $client = $appointment->clients()->first()?->toArray();
        $counselor = $appointment->counselors()->first()?->toArray();

        $appointment->delete();

        if ($appointment->type == 'internal' && $client && $counselor) {
            broadcast((new AppointmentDeletedNotification($appointment_details, $organization, $client, $counselor))->locale(app()->getLocale() ?? 'en'));
        }

        return Redirect::back()->with('success', 'Appointment deleted.');
    }
}
