<?php

namespace App\Http\Controllers\Auth;

use App\Events\ClientRegisteredEvent;
use App\Events\UserCreatedEvent;
use App\Http\Controllers\Controller;
use App\Http\Requests\RegisterUserRequest;
use App\Models\Organization;
use App\Models\User;
use App\Providers\RouteServiceProvider;
use Illuminate\Auth\Events\Registered;
use Illuminate\Support\Facades\Hash;
use Inertia\Inertia;

class RegisteredUserController extends Controller
{
    /**
     * Handle an incoming registration request.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\RedirectResponse
     *
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(RegisterUserRequest $request)
    {
        $validated = $request->validated();

        $user = User::create([
            'first_name' => $validated['first_name'],
            'last_name' => $validated['last_name'],
            'email' => $validated['email'],
            'password' => Hash::make($validated['password']),
            'phone_number' => $validated['phone_number'] ?: NULL,
            'email_verified_at' => $validated['role'] == 'client' ? now() : null,
        ]);

        $organization = Organization::find($validated['organization_id'], ['id', 'uuid']);

        $user->organizations()->attach($validated['organization_id']);
        $user->currentOrganization()->associate($organization);

        switch ($validated['role']) {
            case 'organization-admin':
                $user->syncRoles(['organization-admin']);
                break;

            case 'counselor':
                $user->syncRoles(['counselor']);
                break;

            case 'client':
                $user->syncRoles(['client']);
                if (isset($validated['counselor_id'])) {
                    $counselor = User::find($validated['counselor_id']);
                    if ($counselor) {
                        event(new ClientRegisteredEvent($organization, $user, $counselor, true));
                    }
                }
                break;
        }

        $user->save();

        event(new Registered($user));
        event(new UserCreatedEvent($organization, $user, $validated['role']));

        return redirect(RouteServiceProvider::HOME);
    }

    /**
     * Display the registration view.
     *
     * @return \Inertia\Response
     */
    public function create()
    {
        return Inertia::render('Auth/Register');
    }
}
