<?php

namespace App\Http\Controllers;

use App\Http\Resources\UserResource;
use App\Models\Organization;
use App\Models\User;
use App\Services\ClientService;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Request;
use Inertia\Inertia;

class ClientController extends Controller
{
    public function index(Organization $organization)
    {
        abort_if(!auth()->user()->hasPermissionTo('view_clients'), 403);

        $clients = User::query()
            ->isOrganizationMember($organization)
            ->isClient()
            ->when(!Request::input('all') || Request::input('all') != 1 || !auth()->user()->hasPermissionTo('view_all_clients'), function ($query, $all) {
                $query->counselorClients();
            })
            ->when(Request::input('search'), function ($query, $search) {
                $query->where(function (Builder $query) use ($search) {
                    return $query->where('first_name', 'like', "%{$search}%")
                        ->orWhere('last_name', 'like', "%{$search}%");
                });
            })
            ->select('id', 'first_name', 'last_name', 'email', 'phone_number', 'is_active', 'counselor_id')
            ->with(['counselor:id,first_name,last_name,email,phone_number'])
            ->paginate(10)
            ->withQueryString();

        return Inertia::render('Clients/Index', [
            'organization' => $organization->only('id', 'name'),
            'clients' => UserResource::collection($clients),
            'filters' => Request::only(['search', 'all']),
        ]);
    }

    public function show(Organization $organization, User $user)
    {
        abort_if(!$user->hasRole('client'), 404);

        return Inertia::render('Clients/Show', [
            'user' => $user->load(['organizations:id,name', 'counselor:id,first_name,last_name,email,phone_number', 'conversations:id,uuid', 'appointments:id,type,static_title,description,location,start', 'user_meta']),
            'counselors' => User::query()
                ->isOrganizationMember($organization)
                ->isCounselorOrAdmin()
                ->select('id', 'first_name', 'last_name', 'email', 'phone_number')
                ->get()
                ->map(function ($user) {
                    return [
                        'id' => $user->id,
                        'title' => $user->first_name . ' ' . $user->last_name,
                        'email' => $user->email,
                    ];
                }),
        ]);
    }

    public function edit(Organization $organization, User $user)
    {
        abort_if(!$user->hasRole('client'), 404);

        return Inertia::render('Clients/Edit', [
            'user' => $user->load(['organizations:id,name', 'counselor:id,first_name,last_name,email,phone_number', 'user_meta']),
            'counselors' => User::query()
                ->isOrganizationMember($organization)
                ->isCounselorOrAdmin()
                ->select('id', 'first_name', 'last_name', 'email', 'phone_number')
                ->get()
                ->map(function ($user) {
                    return [
                        'id' => $user->id,
                        'title' => $user->first_name . ' ' . $user->last_name,
                        'email' => $user->email,
                    ];
                }),
        ]);
    }

    public function destroy(Organization $organization, User $user, ClientService $clientService)
    {
        abort_if(!auth()->user()->hasPermissionTo('delete_clients'), 403);

        $clientService->deleteClient($user);

        return redirect()->route('organizations.clients.index', $organization->id)->with('success', 'Deleted successfully.');
    }


    public function change_counselor(Organization $organization, User $user, User $counselor)
    {
        abort_if(!$user->hasRole('client'), 404);
        abort_if(!auth()->user()->hasPermissionTo('edit_clients'), 403);

        $user->counselor()->associate($counselor->id);
        $user->update();

        return redirect()->route('organizations.clients.show', [$organization->id, $user->id])->with('success', 'Counselor changed successfully.');
    }

}
