<?php

namespace App\Http\Controllers;

use App\Events\ConversationAssetDeletedEvent;
use App\Events\ConversationAssetUpdatedEvent;
use App\Events\InviteToConversationAssetEvent;
use App\Events\newConversationAssetEvent;
use App\Events\UpdatedCompetenceCardSetEvent;
use App\Http\Requests\StoreConversationAssetRequest;
use App\Http\Requests\UpdateCompetenceCardSetRequest;
use App\Http\Resources\ConversationAssetResource;
use App\Models\Conversation;
use App\Models\ConversationAsset;
use App\Models\Organization;
use App\Services\CompetenceCardService;
use Inertia\Inertia;

class ConversationAssetController extends Controller
{

    public function store(StoreConversationAssetRequest $request, Organization $organization, Conversation $conversation, CompetenceCardService $competenceCardService)
    {
        abort_if(!auth()->user()->hasPermissionTo('publish_conversation_assets'), 403);

        $validated = $request->validated();

        $conversation_asset = ConversationAsset::create($validated);
        $conversation_asset->conversation()->associate($conversation->id);

        $conversation_asset->save();

        broadcast(new newConversationAssetEvent($conversation_asset));

        return redirect()->route('organizations.conversations.show', [$organization->id, $conversation->uuid, $conversation_asset->id]);

    }

    public function show(Organization $organization, Conversation $conversation, ConversationAsset $conversation_asset)
    {
        abort_if(!auth()->user()->hasPermissionTo('view_conversation_assets'), 403);

        $conversation_asset = new ConversationAssetResource($conversation_asset->load(['conversation', 'user']));
        $conversation_asset->getMedia();

        return Inertia::render('Conversations/Show', [
            'conversation_asset' => $conversation_asset,
        ]);
    }

    public function destroy(Organization $organization, Conversation $conversation, ConversationAsset $conversation_asset)
    {
        abort_if(!auth()->user()->hasPermissionTo('delete_conversation_assets'), 403);
        abort_if(empty($conversation_asset), 404);

        $id = $conversation_asset->id;

        $conversation_asset->delete();

        broadcast(new ConversationAssetDeletedEvent($conversation, $id, 'conversation_asset'));

        return redirect(route('organizations.conversations.show', [$organization->id, $conversation->uuid]));
    }

    public function inviteToConversationAsset(Organization $organization, Conversation $conversation, ConversationAsset $conversation_asset)
    {
        abort_if(!auth()->user()->hasPermissionTo('view_conversation_assets'), 403);
        abort_if(empty($conversation_asset), 404);

        $users_to_invite = $conversation->users()
            ->select(['id', 'first_name', 'last_name', 'email', 'phone_number'])
            ->where('id', '!=', auth()->user()->id)
            ->get()
            ->pluck('id')
            ->toArray();

        broadcast(new InviteToConversationAssetEvent($conversation_asset, $users_to_invite));

        return redirect()->back()->with('success', 'Invites sent successfully');
    }

    public function updateCompetenceCardSet(UpdateCompetenceCardSetRequest $request, Organization $organization, Conversation $conversation, ConversationAsset $conversationAsset)
    {
        abort_if(!auth()->user()->hasPermissionTo('edit_conversation_assets'), 403);

        $conversationAsset->update($request->validated());

        broadcast(new UpdatedCompetenceCardSetEvent($conversationAsset));

        return redirect()->back();
    }

    public function update(Organization $organization, Conversation $conversation, StoreConversationAssetRequest $request, ConversationAsset $conversation_asset)
    {
        abort_if(!auth()->user()->hasPermissionTo('edit_conversation_assets'), 403);
        abort_if(empty($conversation_asset), 404);

        $conversation_asset->update($request->validated());

        broadcast(new ConversationAssetUpdatedEvent($conversation_asset));

        return redirect()->back();
    }
}
