<?php

namespace App\Http\Controllers;

use App\Events\ConversationAssetDeletedEvent;
use App\Events\ConversationMediaUpdatedEvent;
use App\Events\KnowledgebaseArticleCreatedEvent;
use App\Http\Requests\AddMediaToConversationRequest;
use App\Http\Requests\RemoveMediaToConversationRequest;
use App\Http\Resources\CompetenceCardResource;
use App\Http\Resources\ConversationAssetResource;
use App\Http\Resources\ConversationResource;
use App\Http\Resources\MessageResource;
use App\Models\CompetenceCard;
use App\Models\Conversation;
use App\Models\ConversationAsset;
use App\Models\KnowledgebaseArticle;
use App\Models\Organization;
use App\Services\MessageService;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Request;
use Inertia\Inertia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class ConversationController extends Controller
{
    public function index(Organization $organization)
    {
        $conversations = Conversation::query()
            ->whereBelongsTo($organization)
            ->when(!Request::input('all') || Request::input('all') != 1 || !auth()->user()->hasPermissionTo('view_all_conversations'), function ($query, $all) {
                $query->belongsToUser();
            })
            ->when(Request::input('search'), function ($query, $search) {
                $query->whereHas('users', function (Builder $query) use ($search) {
                    return $query->where('first_name', 'like', "%{$search}%")
                        ->orWhere('last_name', 'like', "%{$search}%");
                });
            })
            ->select('id', 'uuid', 'description', 'organization_id', 'is_active')
            ->with('users:id,first_name,last_name,email,phone_number')
            ->with('appointments', function ($query) {
                $query->whereDate('start', '>=', now()->toDateString())->first();
            })
            ->paginate(12)
            ->withQueryString();

        return Inertia::render('Conversations/Index', [
            'conversations' => $conversations,
            'filters' => Request::only(['all', 'search']),
        ]);
    }

    public function show(Organization $organization, Conversation $conversation, ConversationAsset $conversation_asset = null)
    {

        if ($conversation_asset && $conversation_asset->type == 'timeline') {
            $conversation_asset->load(['timeline_items']);
        }

        if ($conversation_asset && $conversation_asset->type == 'competence_card_set') {
            $competenceCards = CompetenceCardResource::collection(CompetenceCard::isActive()->with('competence_card_category')->get());
        }

        $conversation_media = $conversation->getMedia('organization-' . $organization->id . '-conversation-media-' . $conversation->id);

        $users = $conversation->users()
            ->select('id', 'first_name', 'last_name', 'email', 'phone_number')
            ->with('roles:id,name')
            ->get();

        $users = $users->map(function ($user) {
            unset($user->email);
            unset($user->phone_number);
            return $user;
        });

        $appointments = $conversation->appointments()
            ->select(['id', 'organization_id', 'conversation_id', 'static_title', 'type', 'location', 'description', 'start', 'end', 'status', 'appointment_type'])
            ->orderBy('start')
            ->with(['users:id,first_name,last_name,email,phone_number', 'conversation:id,uuid,organization_id'])
            ->get();

        return Inertia::render('Conversations/Show', [
            'conversation' => new ConversationResource($conversation),
            'users' => $users,
            'messages' => MessageResource::collection($conversation->messages()
                ->with('sender')
                ->orderby('created_at', 'ASC')
                ->limit(100)
                ->get()),
            'appointments' => $appointments ?? null,
            'knowledgebase_articles' => $conversation->knowledgebase_articles ?? null,
            'conversation_assets' => $conversation->conversation_assets ?? null,
            'conversation_asset' => $conversation_asset ? new ConversationAssetResource($conversation_asset) : null,
            'conversation_media' => $conversation_media ?? null,
            'competence_cards' => $competenceCards ?? null,
            'conversation_notes' => auth()->user()->hasAnyRole(['counselor', 'organization-admin', 'super-admin']) && $conversation->conversation_notes ? $conversation->conversation_notes : null,
            'shareable_knowledgebase_articles' => auth()->user()->hasAnyPermission(['edit_all_conversations', 'edit_conversations']) ? Inertia::lazy(fn() => KnowledgebaseArticle::isAttachable()->select(['id', 'title'])->get()) : [],
        ]);
    }

    public function add_media(AddMediaToConversationRequest $request, Organization $organization, Conversation $conversation, MessageService $messageService)
    {
        $validated = $request->validated();

        $conversation
            ->addFromMediaLibraryRequest($validated['conversation_media'])
            ->toMediaCollection('organization-' . $organization->id . '-conversation-media-' . $conversation->id);

        $file_links = [];

        if (!empty($validated['conversation_media'])) {

            foreach ($validated['conversation_media'] as $requestMediaItem) {
                $mediaItem = Media::where('uuid', $requestMediaItem['uuid'])->first();
                $mediaItem->setCustomProperty('organization_id', $organization->id);
                $mediaItem->setCustomProperty('user_id', auth()->user()->id);
                $mediaItem->setCustomProperty('media_type', 'conversation_media');
                $mediaItem->setCustomProperty('conversation_uuid', $conversation->uuid);
                $mediaItem->save();
                $file_links[] = '<a href="' . route('userMedia', ['organization' => $organization, 'mediaId' => $mediaItem->id, 'fileName' => $mediaItem->file_name]) . '" target="_blank" download>' . $mediaItem->file_name . '</a> ';
            }
        }

        broadcast(new ConversationMediaUpdatedEvent($conversation));

        $message = count($file_links) > 1 ? count($file_links) . ' ' . __('files sent') . ': ' : count($file_links) . ' ' . __('file sent') . ': ';
        $message .= implode(', ', $file_links);

        $messageService->createMessage($message, $conversation, $organization, 'file');

        return redirect()->back()->with('success', 'File(s) added to conversation');
    }

    public function remove_media(RemoveMediaToConversationRequest $request, Organization $organization, Conversation $conversation)
    {
        $validated = $request->validated();

        $media_to_delete = $conversation
            ->getMedia('organization-' . $organization->id . '-conversation-media-' . $conversation->id)
            ->where('uuid', $validated['uuid']);

        if ($media_to_delete->isNotEmpty()) {
            $media_to_delete->first()->delete();
        }

        broadcast(new ConversationMediaUpdatedEvent($conversation));

        return redirect()->back()->with('success', 'File deleted');
    }

    public function attach_knowldegebase_article(Organization $organization, Conversation $conversation, KnowledgebaseArticle $knowledgebase_article, MessageService $messageService)
    {
        if ($conversation->knowledgebase_articles()->where('id', $knowledgebase_article->id)->exists()) {
            return redirect()->back()->with('error', 'Article is already attached to conversation');
        }

        $conversation->knowledgebase_articles()->attach($knowledgebase_article->id);

        broadcast(new KnowledgebaseArticleCreatedEvent($conversation, $knowledgebase_article));

        $message = __('Article shared') . ': <a href="' . route('organizations.knowledgebase_articles.show', [$organization->id, $knowledgebase_article->id]) . '" target="_blank">' . $knowledgebase_article->title . '</a>';
        $messageService->createMessage($message, $conversation, $organization, 'article');

        return redirect()->back()->with('success', 'Article was added to conversation');
    }

    public function detach_knowledgebase_article(Organization $organization, Conversation $conversation, KnowledgebaseArticle $knowledgebase_article)
    {
        $conversation->knowledgebase_articles()->detach($knowledgebase_article->id);

        broadcast(new ConversationAssetDeletedEvent($conversation, $knowledgebase_article->id, 'knowledgebase_article'));

        return redirect()->back()->with('success', 'Article was removed from conversation');
    }

}
