<?php

namespace App\Http\Controllers;

use App\Http\Requests\StoreKnowledgebaseArticleRequest;
use App\Http\Requests\UpdateKnowledgebaseArticleRequest;
use App\Models\KnowledgebaseArticle;
use App\Models\KnowledgebaseCategory;
use App\Models\Organization;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Request;
use Inertia\Inertia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class KnowledgebaseArticleController extends Controller
{
    public function __construct()
    {
        $this->middleware('permission:view_knowledgebase_articles|view_all_knowledgebase_articles')->only(['index', 'show']);
        $this->middleware('permission:edit_knowledgebase_articles|edit_all_knowledgebase_articles')->only(['edit', 'update']);
        $this->middleware('permission:publish_knowledgebase_articles|publish_all_knowledgebase_articles')->only(['create', 'store']);
        $this->middleware('permission:publish_knowledgebase_articles|publish_all_knowledgebase_articles')->only(['create', 'store']);
        $this->middleware('permission:delete_knowledgebase_articles|delete_all_knowledgebase_articles')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Inertia\Response
     */
    public function index(Organization $organization)
    {

        return Inertia::render('KnowledgebaseArticles/Index', [
            'knowledgebase_articles' => KnowledgebaseArticle::query()
                ->whereBelongsTo($organization)
                ->belongsToUser()
                ->when(Request::input('search'), function ($query, $search) {
                    $query->where('title', 'like', "%{$search}%");
                })
                ->when(Request::input('category_id'), function ($query, $category_id) {
                    $query->whereHas('categories', function ($query) use ($category_id) {
                        $query->where('id', $category_id);
                    });
                })
                ->with(['organization', 'categories'])
                ->orderBy('created_at', 'desc')
                ->paginate(18)
                ->withQueryString(),
            'knowledgebase_categories' => KnowledgebaseCategory::query()
                ->whereBelongsTo($organization)
                ->get(),
            'filters' => Request::only('search', 'category_id'),
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(Organization $organization, StoreKnowledgebaseArticleRequest $request)
    {
        $validated = $request->validated();
        $article = KnowledgebaseArticle::create($validated);

        $article
            ->syncFromMediaLibraryRequest($validated['knowledgebase_article_media'])
            ->toMediaCollection('organization-' . $organization->id . '-knowledgebase-article-media-' . $article->id);

        $article->categories()->sync($validated['knowledgebase_category']);
        $article->save();

        if (!empty($validated['knowledgebase_article_media'])) {
            foreach ($validated['knowledgebase_article_media'] as $requestMediaItem) {
                $mediaItem = Media::where('uuid', $requestMediaItem['uuid'])->first();
                $mediaItem->setCustomProperty('organization_id', $organization->id);
                $mediaItem->setCustomProperty('user_id', auth()->user()->id);
                $mediaItem->save();
            }
        }

        return Redirect::route('organizations.knowledgebase_articles.show', ['organization' => $organization, 'knowledgebase_article' => $article])->with('success', 'Article created.');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Inertia\Response
     */
    public function create(Organization $organization)
    {
        return Inertia::render('KnowledgebaseArticles/Create', [
            'knowledgebase_categories' => KnowledgebaseCategory::query()
                ->whereBelongsTo($organization)
                ->get(),
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param KnowledgebaseArticle $knowledgebaseArticle
     * @return \Inertia\Response
     */
    public function show(Organization $organization, KnowledgebaseArticle $knowledgebaseArticle)
    {

        $knowledgebaseArticle->getMedia('knowledgebase-article-media');

        return Inertia::render('KnowledgebaseArticles/Show', [
            'knowledgebase_article' => $knowledgebaseArticle->load('categories'),
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param KnowledgebaseArticle $knowledgebaseArticle
     * @return \Inertia\Response
     */
    public function edit(Organization $organization, KnowledgebaseArticle $knowledgebaseArticle)
    {
        $knowledgebaseArticle->getMedia();

        return Inertia::render('KnowledgebaseArticles/Edit', [
            'knowledgebase_article' => $knowledgebaseArticle->load('categories'),
            'knowledgebase_categories' => KnowledgebaseCategory::all(),
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param KnowledgebaseArticle $knowledgebaseArticle
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Organization $organization, KnowledgebaseArticle $knowledgebaseArticle, UpdateKnowledgebaseArticleRequest $request)
    {
        $validated = $request->validated();

        $knowledgebaseArticle->update($validated);

        $knowledgebaseArticle
            ->syncFromMediaLibraryRequest($request->knowledgebase_article_media)
            ->toMediaCollection('organization-' . $organization->id . '-knowledgebase-article-media-' . $knowledgebaseArticle->id);

        $knowledgebaseArticle->categories()->sync($validated['knowledgebase_category']);
        $knowledgebaseArticle->save();

        if (!empty($validated['knowledgebase_article_media'])) {
            foreach ($validated['knowledgebase_article_media'] as $requestMediaItem) {
                $mediaItem = Media::where('uuid', $requestMediaItem['uuid'])->first();
                $mediaItem->setCustomProperty('organization_id', $organization->id);
                $mediaItem->setCustomProperty('user_id', auth()->user()->id);
                $mediaItem->save();
            }
        }

        return Redirect::route('organizations.knowledgebase_articles.show', ['organization' => $organization, 'knowledgebase_article' => $knowledgebaseArticle])->with('success', 'Knowledgebase Article updated.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param KnowledgebaseArticle $knowledgebaseArticle
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy(Organization $organization, KnowledgebaseArticle $knowledgebaseArticle)
    {
        if ($knowledgebaseArticle) {
            $knowledgebaseArticle->delete();
            return Redirect::route('organizations.knowledgebase_articles.index', ['organization' => $organization])->with('success', 'Knowledgebase Article deleted.');
        }
    }
}
