<?php

namespace App\Http\Controllers;

use App\Http\Requests\InviteUserRequest;
use App\Http\Requests\UpdateOrganizationRequest;
use App\Models\Organization;
use App\Models\User;
use App\Services\MediaService;
use App\Services\OrganizationService;
use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Request;
use Inertia\Inertia;
use Spatie\Permission\Models\Role;

class OrganizationController extends Controller
{

    public function __construct()
    {
        $this->middleware('permission:view_organizations|view_all_organizations')->only(['index', 'show']);
        $this->middleware('permission:edit_organizations|edit_all_organizations')->only(['edit', 'update']);
        $this->middleware('permission:publish_organizations|publish_all_organizations')->only(['create', 'store']);
        $this->middleware('permission:publish_organizations|publish_all_organizations')->only(['create', 'store']);
        $this->middleware('permission:delete_organizations|delete_all_organizations')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Inertia\Response
     */
    public function index()
    {
        $organizations = Organization::query()
            ->when(!Request::input('all') || Request::input('all') != 1 || !auth()->user()->hasPermissionTo('view_all_conversations'), function ($query, $all) {
                $query->belongsToUser();
            })
            ->when(Request::input('search'), function ($query, $search) {
                $query->where('name', 'like', "%{$search}%");
            })
            ->select('id', 'name', 'phone_number', 'email', 'website', 'is_active')
            ->withCount('users')
            ->paginate(20)
            ->withQueryString();

        return Inertia::render('Organizations/Index', [
            'organizations' => $organizations,
            'filters' => Request::only(['all', 'search']),
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(UpdateOrganizationRequest $request)
    {

        $validated = $request->validated();
        $organization = Organization::create($validated);

        $this->saveOrganizationLogo($organization, $validated);

        return Redirect::route('organizations.index')->with('success', 'Organization created.');

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Inertia\Response
     */
    public function create()
    {
        abort_if(!auth()->user()->hasRole('super-admin'), 403);

        return Inertia::render('Organizations/Create');
    }

    /**
     * Display the specified resource.
     *
     * @param $id
     * @return \Inertia\Response
     */
    public function show(Organization $organization)
    {

        $organization_admins = $organization->organization_admins()
            ->with('roles')
            ->select('id', 'first_name', 'last_name', 'email', 'phone_number', 'is_active', 'email_verified_at')
            ->get();

        $organization_employees = $organization->organization_employees()
            /*->when(Request::input('search_employee'), function ($query, $search_employee) {
                $query->where(function (Builder $query) use ($search_employee) {
                    return $query->whereBlind('first_name', 'first_name_index', "%{$search_employee}%")
                        ->orWhereBlind('last_name', 'last_name_index', "%{$search_employee}%");
                });
            })*/
            ->with('roles')
            ->select('id', 'first_name', 'last_name', 'email', 'phone_number', 'is_active', 'email_verified_at')
            ->get();

        $organization_clients = $organization->organization_clients()
            ->with('counselor:id,first_name,last_name,email,phone_number')
            ->select('id', 'first_name', 'last_name', 'email', 'phone_number', 'is_active', 'counselor_id')
            ->get();

        /*if (Request::input('search_client')) {
            $client_search_term = Request::input('search_client');

            $organization_clients = $organization_clients->filter(function ($item) use ($client_search_term) {
                $full_name = $item['first_name'] . ' ' . $item['last_name'];
                return stripos($full_name, $client_search_term) !== false;
            });
        }*/

        $organization->getMedia('organization-logo-' . $organization->id);

        $assignable_roles = Role::where('name', '=', 'organization-admin')
            ->orWhere('name', '=', 'counselor')->get();

        return Inertia::render('Organizations/Show', [
            'organization' => $organization,
            'organization_admins' => $organization_admins,
            'organization_employees' => $organization_employees,
            'organization_clients' => $organization_clients,
            'roles' => $assignable_roles,
            'filters' => Request::only(['search_employee', 'search_client']),
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param $id
     * @return \Inertia\Response
     */
    public function edit(Organization $organization)
    {
        $organization->getMedia();

        return Inertia::render('Organizations/Edit', [
            'organization' => $organization,
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(UpdateOrganizationRequest $request, Organization $organization, MediaService $mediaService)
    {
        $validated = $request->validated();
        $organization->update($validated);

        $mediaService->saveOrganizationLogo($organization, $validated);

        return Redirect::route('organizations.index')->with('success', 'Organization updated.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy(Organization $organization)
    {
        $organization->delete();
        return redirect()->back();
    }

    public function inviteUser(InviteUserRequest $request, Organization $organization, OrganizationService $organizationService)
    {
        $validated = $request->validated();

        abort_if(!auth()->user()->hasPermissionTo('invite_counselors') && $validated['role'] == 'counselor', 403, 'You are not allowed to invite counselors.');
        abort_if(!auth()->user()->hasPermissionTo('invite_clients') && $validated['role'] == 'client', 403, 'You are not allowed to invite clients to the organization.');

        $recipient = [$validated['email'] => $validated['first_name'] . ' ' . $validated['last_name']];

        $counselor = User::find($validated['counselor_id'], ['id', 'first_name', 'last_name', 'email', 'phone_number']);
        $organization = Organization::find($validated['organization_id'], ['id', 'uuid', 'name']);

        $invite_url = $organizationService->generateUserInviteUrl(
            $validated['email'],
            $validated['first_name'],
            $validated['last_name'],
            $organization->id,
            $organization->uuid,
            $counselor->id,
            $validated['role'],
            $counselor,
            $organization
        );

        if ($validated['send_email']) {
            Notification::route('mail', $recipient)
                ->notify((new \App\Notifications\InviteEmployeeNotification($validated, $invite_url, $organization, $counselor))->locale(app()->getLocale() ?? 'en'));
        }

        return redirect()->back()->with('success', $invite_url);
    }

    public function join_organization($organization_id)
    {
        abort_if(!auth()->user()->hasRole('super-admin'), 403);

        $organization = Organization::findOrFail($organization_id);

        abort_if($organization->users->contains(auth()->user()), 403, 'You are already part of this organization.');

        $organization->users()->attach(auth()->user());
        $organization->save();

        return redirect()->route('organizations.index');
    }

    public function change_organization($organization_id)
    {

        $organization = Organization::findOrFail($organization_id);
        abort_if(!$organization->users()->where('users.id', auth()->user()->id)->exists(), 403);

        $user = auth()->user();
        $user->currentOrganization()->associate($organization_id);
        $user->save();

        session(['organization_id' => $organization->id, 'organization_name' => $organization->name]);

        return redirect()->route('organization.dashboard')->with('success', 'Organization changed.');
    }

    public function settings()
    {
        return Inertia::render('Settings/Settings', [
            'organization' => auth()->user()->currentOrganization,
        ]);
    }
}
