<?php

namespace App\Http\Controllers;

use App\Events\ClientRegisteredEvent;
use App\Events\UserCreatedEvent;
use App\Http\Requests\StoreUserRequest;
use App\Http\Requests\UpdatePasswordRequest;
use App\Http\Requests\UpdateProfileRequest;
use App\Http\Resources\OrganizationResource;
use App\Models\Organization;
use App\Models\User;
use App\Services\RoleService;
use Illuminate\Auth\Events\Registered;
use Illuminate\Support\Facades\Hash;
use Inertia\Inertia;
use Spatie\Permission\Models\Role;

class UserController extends Controller
{
    public function store(Organization $organization, StoreUserRequest $request)
    {
        $validated = $request->validated();
        $role = Role::find($validated['role_id']);

        abort_if(
            ($role->name == 'organization-admin' && !auth()->user()->hasPermissionTo('publish_all_users'))
            || ($role->name == 'counselor' && !auth()->user()->hasPermissionTo('publish_all_users'))
            || ($role->name == 'client' && !auth()->user()->hasPermissionTo('publish_clients'))
            , 403);

        $user = User::create([
            'first_name' => $validated['first_name'],
            'last_name' => $validated['last_name'],
            'email' => $validated['email'],
            'password' => Hash::make($validated['password']),
            'phone_number' => $validated['phone_number'] ?: NULL,
            'email_verified_at' => $role->name == 'client' ? now() : null,
        ]);

        $user->organizations()->attach($organization->id);
        $user->currentOrganization()->associate($organization->id);

        switch ($role->name) {
            case 'organization-admin':
                abort_if(!auth()->user()->hasPermissionTo('publish_all_users'), 403);
                $user->syncRoles(['organization-admin']);
                break;

            case 'counselor':
                abort_if(!auth()->user()->hasPermissionTo('publish_all_users'), 403);
                $user->syncRoles(['counselor']);
                break;

            case 'client':
                abort_if(!auth()->user()->hasPermissionTo('publish_clients'), 403);
                $user->syncRoles(['client']);

                if (isset($validated['counselor_id'])) {
                    $counselor = User::find($validated['counselor_id']);
                    if ($counselor) {
                        event(new ClientRegisteredEvent($organization, $user, $counselor, false));
                    }
                }
                break;
        }

        $user->save();

        event(new UserCreatedEvent($organization, $user, $role->name, auth()->user(), $validated['password']));

        if ($role->name == 'organization-admin' || $role->name == 'counselor') {
            event(new Registered($user));
        }

        return auth()->user()->hasRole('counselor') && $role->name == 'client'
            ? redirect()->route('organizations.clients.index', [$organization->id])->with('success', 'Client created successfully')
            : redirect()->route('organizations.show', [$organization->id])->with('success', 'User created successfully');
    }

    public function create(Organization $organization, RoleService $roleService)
    {
        abort_if(!auth()->user()->hasPermissionTo('publish_all_users') && !auth()->user()->hasPermissionTo('publish_clients'), 403);

        $roles = $roleService->getAvailableRolesForUser(auth()->user());

        $counselors = User::query()
            ->isOrganizationMember($organization)
            ->isCounselorOrAdmin()
            ->select('id', 'first_name', 'last_name', 'email', 'phone_number')
            ->get()
            ->map(function ($user) {
                return [
                    'id' => $user->id,
                    'title' => $user->first_name . ' ' . $user->last_name,
                    'email' => $user->email,
                ];
            });

        return Inertia::render('Organizations/CreateUser', [
            'organization' => new OrganizationResource($organization),
            'roles' => $roles,
            'counselors' => $counselors,
        ]);

    }

    public function edit(Organization $organization, User $user)
    {
        return Inertia::render('Settings/EditProfile', [
            'user' => auth()->user()->load(['organizations:id,name']),
        ]);
    }

    public function update_profile(UpdateProfileRequest $request, Organization $organization, User $user)
    {
        abort_if($user->id != auth()->user()->id, 403);

        $validated = $request->validated();
        $user->update($validated);

        return redirect()->route('organization.dashboard')->with('success', 'Profile updated successfully');
    }

    public function update_password(UpdatePasswordRequest $request, Organization $organization, User $user)
    {
        abort_if($user->id != auth()->user()->id, 403);

        $validated = $request->validated();
        $user->update([
            'password' => Hash::make($validated['password']),
        ]);

        return redirect()->route('organization.dashboard')->with('success', 'Password updated successfully');
    }
}
