<?php

namespace App\Models;

use App\Traits\Tenantable;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use ParagonIE\CipherSweet\BlindIndex;
use ParagonIE\CipherSweet\EncryptedRow;
use Spatie\LaravelCipherSweet\Concerns\UsesCipherSweet;
use Spatie\LaravelCipherSweet\Contracts\CipherSweetEncrypted;
use SpatiePermissionsToVueInertia\Traits\SpatiePermissionsToVue;

class Appointment extends Model implements CipherSweetEncrypted
{
    use SoftDeletes, SpatiePermissionsToVue, UsesCipherSweet, Tenantable, hasFactory;

    public $appends = ['ModelRoutes', 'title', 'start_date'];

    /* Appends */
    protected $fillable = [
        'organization_id',
        'conversation_id',
        'static_title',
        'description',
        'type',
        'location',
        'attendees',
        'start',
        'end',
        'status',
        'appointment_type'
    ];
    protected $casts = [
        'attendees' => 'array',
    ];

    public static function configureCipherSweet(EncryptedRow $encryptedRow): void
    {

        $encryptedRow
            ->addField('static_title')
            ->addBlindIndex('static_title', new BlindIndex('static_title_index'));

        $encryptedRow
            ->addField('description')
            ->addBlindIndex('description', new BlindIndex('description_index'));

        $encryptedRow
            ->addField('location')
            ->addBlindIndex('location', new BlindIndex('location_index'));

    }

    public function getModelRoutesAttribute()
    {
        if (!auth()->check()) return ['index', 'show', 'edit', 'destroy'];

        return [
            'index' => route('organizations.appointments.index', [auth()->user()->currentOrganization->id]),
            'show' => route('organizations.appointments.show', [auth()->user()->currentOrganization->id, $this->id]),
            'edit' => route('organizations.appointments.edit', [auth()->user()->currentOrganization->id, $this->id]),
            'destroy' => route('organizations.appointments.destroy', [auth()->user()->currentOrganization->id, $this->id]),
        ];
    }

    public function getTitleAttribute()
    {
        return $this->users && $this->type == 'internal' ? implode(', ', $this->users->pluck('full_name')->toArray()) : $this->static_title;
    }

    public function getStartDateAttribute()
    {
        return Carbon::createFromFormat('Y-m-d H:i:s', $this->start)->format('d.m.Y');
    }

    /* Scopes */

    public function scopeBelongsToUser($query)
    {
        return $query->whereHas('users', function ($query) {
            $query->whereIn('user_id', [auth()->user()->id]);
        });
    }

    /* Relations */

    public function organization(): BelongsTo
    {
        return $this->belongsTo(Organization::class);
    }

    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    public function clients(): BelongsToMany
    {
        return $this->belongsToMany(User::class)->whereHas('roles', function ($query) {
            $query->where('name', 'client');
        });
    }

    public function counselors(): BelongsToMany
    {
        return $this->belongsToMany(User::class)->whereHas('roles', function ($query) {
            $query->where('name', 'counselor')
                ->orWhere('name', 'organization-admin')
                ->orWhere('name', 'super-admin');
        });
    }

    public function conversation(): BelongsTo
    {
        return $this->belongsTo(Conversation::class);
    }

    public function conversation_notes(): HasMany
    {
        return $this->hasMany(ConversationNote::class);
    }
}
