<?php

namespace App\Models;

use App\Traits\Tenantable;
use Dyrynda\Database\Support\BindsOnUuid;
use Dyrynda\Database\Support\GeneratesUuid;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasManyThrough;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\Image\Manipulations;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class Conversation extends Model implements HasMedia
{
    use BindsOnUuid, GeneratesUuid, HasFactory, InteractsWithMedia, SoftDeletes, Tenantable;

    public $appends = ['ModelRoutes', 'title'];

    /* Appends */
    protected $fillable = [
        'organization_id',
        'title',
        'description',
        'is_active',
    ];

    public function getModelRoutesAttribute()
    {
        if (!auth()->check()) return ['index', 'show', 'destroy'];

        return [
            'index' => route('organizations.conversations.index', [auth()->user()->currentOrganization->id]),
            'show' => route('organizations.conversations.show', [auth()->user()->currentOrganization->id, $this->uuid]),
            'destroy' => route('organizations.conversations.destroy', [auth()->user()->currentOrganization->id, $this->uuid]),
        ];
    }

    public function registerMediaConversions(Media $media = null): void
    {
        $this
            ->addMediaConversion('preview')
            ->fit(Manipulations::FIT_CROP, 300, 300)
            ->nonQueued();
    }

    /* Scopes */

    public function scopeBelongsToUser($query)
    {
        return $query->whereHas('users', function ($query) {
            $query->whereIn('user_id', [auth()->user()->id]);
        });
    }

    /* Attributes */

    public function getTitleAttribute()
    {
        $other_related_users = $this->users()
            ->whereNotIn('id', [auth()->user()->id])
            ->select(['id', 'first_name', 'last_name', 'email', 'phone_number'])
            ->get();

        return !empty($other_related_users) ? implode(', ', $other_related_users->pluck('full_name')->toArray()) : $this->uuid;
    }

    /* Relationships */

    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    public function clients(): BelongsToMany
    {
        return $this->belongsToMany(User::class)->whereHas('roles', function ($query) {
            $query->where('name', 'client');
        });
    }

    public function organization(): BelongsTo
    {
        return $this->belongsTo(Organization::class);
    }

    public function messages(): HasMany
    {
        return $this->hasMany(Message::class);
    }

    public function appointments(): HasMany
    {
        return $this->hasMany(Appointment::class);
    }

    public function conversation_assets(): HasMany
    {
        return $this->hasMany(ConversationAsset::class);
    }

    public function timelines(): HasMany
    {
        return $this->hasMany(ConversationAsset::class)
            ->where('type', 'timeline')->with(['timeline_items' => function ($query) {
                $query->orderBy('date');
            }]);
    }

    public function competence_card_sets(): HasMany
    {
        return $this->hasMany(ConversationAsset::class)->where('type', 'competence_card_set');
    }

    public function timeline_items(): HasManyThrough
    {
        return $this->hasManyThrough(TimelineItem::class, ConversationAsset::class);
    }

    public function knowledgebase_articles(): BelongsToMany
    {
        return $this->belongsToMany(KnowledgebaseArticle::class, 'conversation_knowledgebase_article', 'conversation_id', 'article_id')->select(['id', 'title', 'organization_id']);
    }

    public function conversation_notes(): HasMany
    {
        return $this->hasMany(ConversationNote::class);
    }

}
