<?php

namespace App\Models;

use App\Traits\Tenantable;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Carbon;
use Spatie\Image\Manipulations;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class KnowledgebaseArticle extends Model implements HasMedia
{
    use HasFactory, SoftDeletes, InteractsWithMedia, Tenantable;

    public $appends = ['ModelRoutes'];
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'title',
        'content',
        'organization_id',
        'shareable'
    ];

    public function getModelRoutesAttribute()
    {
        if (!auth()->check()) return ['index', 'show', 'edit', 'destroy'];

        return [
            'index' => route('organizations.knowledgebase_articles.index', [auth()->user()->currentOrganization->id]),
            'show' => route('organizations.knowledgebase_articles.show', [auth()->user()->currentOrganization->id, $this->id]),
            'edit' => route('organizations.knowledgebase_articles.edit', [auth()->user()->currentOrganization->id, $this->id]),
            'destroy' => route('organizations.knowledgebase_articles.destroy', [auth()->user()->currentOrganization->id, $this->id]),
        ];
    }

    public function registerMediaConversions(Media $media = null): void
    {
        $this
            ->addMediaConversion('preview')
            ->fit(Manipulations::FIT_CROP, 300, 300)
            ->nonQueued();
    }


    public function getUpdatedAtAttribute($value)
    {
        return (new Carbon($value))->format('d.m.Y @ H:i');
    }

    public function organization(): BelongsTo
    {
        return $this->belongsTo(Organization::class);
    }

    public function categories(): BelongsToMany
    {
        return $this->belongsToMany(KnowledgebaseCategory::class, 'knowledgebase_article_knowledgebase_category', 'article_id', 'category_id');
    }

    public function conversations(): BelongsToMany
    {
        return $this->belongsToMany(Conversation::class, 'conversation_knowledgebase_article', 'article_id', 'conversation_id');
    }

    public function scopeIsShareable($query)
    {
        return $query->where('shareable', true);
    }

    public function scopeIsAttachable($query)
    {
        // where not attached to currenct conversation
        return $query
            ->where('shareable', true)
            ->whereDoesntHave('conversations', function ($query) {
                $query->where('conversation_id', request()->conversation->id);
            });
    }


    // scope for articles to only show those which are shared with user in one of the conversations that he belongs to
    public function scopeBelongsToUser($query)
    {

        if (auth()->user()->hasRole('counselor') || auth()->user()->hasRole('organization-admin') || auth()->user()->hasRole('super-admin')) {
            return $query;
        }

        return $query->whereHas('conversations', function ($query) {
            $query->whereHas('users', function ($query) {
                $query->whereIn('user_id', [auth()->user()->id]);
            });
        });
    }

}
