<?php

namespace App\Models;

use App\Traits\Uuids;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\Image\Manipulations;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class Organization extends Model implements HasMedia
{
    use HasFactory, Uuids, SoftDeletes, InteractsWithMedia;

    public $appends = ['ModelRoutes'];
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'uuid',
        'name',
        'street',
        'zip_code',
        'city',
        'country',
        'tax_id',
        'phone_number',
        'email',
        'website',
        'logo_path',
        'appointment_types',
        'is_active',
    ];

    /**
     * The "booted" method of the model.
     *
     * @return void
     */
    protected static function booted()
    {

        if (auth()->user() && !auth()->user()->hasPermissionTo('view_all_organizations')) {
            $user_id = auth()->user()->id;
            static::addGlobalScope('organizations', function (Builder $builder) use ($user_id) {
                $builder->whereHas('users', function ($query) use ($user_id) {
                    $query->where('organization_user.user_id', $user_id);
                });
            });
        }

    }

    public function getModelRoutesAttribute()
    {
        if (!auth()->check()) return ['index', 'show', 'edit', 'destroy'];

        return [
            'index' => $this->id ? route('organizations.index') : NULL,
            'show' => $this->id ? route('organizations.show', ['organization' => $this->id]) : NULL,
            'edit' => $this->id ? route('organizations.edit', ['organization' => $this->id]) : NULL,
            'destroy' => $this->id ? route('organizations.destroy', ['organization' => $this->id]) : NULL,
        ];
    }

    public function registerMediaConversions(Media $media = null): void
    {
        $this
            ->addMediaConversion('preview')
            ->fit(Manipulations::FIT_CROP, 300, 300)
            ->nonQueued();
    }


    public function users()
    {
        return $this->BelongsToMany(User::class);
    }

    public function organization_employees()
    {
        return $this->BelongsToMany(User::class)->role(['organization-admin', 'counselor']);
    }

    public function organization_clients()
    {
        return $this->BelongsToMany(User::class)->role(['client']);
    }

    public function counselors()
    {
        return $this->BelongsToMany(User::class)->role('counselor');
    }

    public function organization_admins()
    {
        return $this->BelongsToMany(User::class)->role('organization-admin');
    }

    public function knowledgeBaseArticles()
    {
        return $this->HasMany(KnowledgebaseArticle::class);
    }

    public function knowledgeBaseCategories()
    {
        return $this->HasMany(KnowledgebaseCategory::class);
    }

    public function appointments()
    {
        return $this->hasMany(Appointment::class);
    }

    /* Scopes */

    public function scopeBelongsToUser($query)
    {
        return $query->whereHas('users', function ($query) {
            $query->whereIn('user_id', [auth()->user()->id]);
        });
    }

    public function scopeActive($query)
    {
        $query->where('is_active', 1);
    }

}
