<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Route;
use ParagonIE\CipherSweet\BlindIndex;
use ParagonIE\CipherSweet\EncryptedRow;
use Spatie\LaravelCipherSweet\Concerns\UsesCipherSweet;
use Spatie\LaravelCipherSweet\Contracts\CipherSweetEncrypted;
use Spatie\Permission\Traits\HasRoles;
use SpatiePermissionsToVueInertia\Traits\SpatiePermissionsToVue;

class User extends Authenticatable implements MustVerifyEmail, CipherSweetEncrypted
{
    use HasFactory, Notifiable, SoftDeletes, SpatiePermissionsToVue, HasRoles, UsesCipherSweet;

    public $appends = ['ModelRoutes', 'full_name'];
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
        'last_name',
        'email',
        'password',
        'phone_number',
        'current_organization_id',
        'is_active',
        'email_verified_at',
        'last_online_at'
    ];
    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];
    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'is_active' => 'boolean',
    ];

    /* Appends */

    public static function configureCipherSweet(EncryptedRow $encryptedRow): void
    {
        $encryptedRow
            ->addField('email')
            ->addBlindIndex('email', new BlindIndex('email_index'));

        $encryptedRow
            ->addField('first_name')
            ->addBlindIndex('first_name', new BlindIndex('first_name_index'));

        $encryptedRow
            ->addField('last_name')
            ->addBlindIndex('last_name', new BlindIndex('last_name_index'));

        $encryptedRow
            ->addField('phone_number')
            ->addBlindIndex('phone_number', new BlindIndex('phone_number_index'));

    }

    public static function getEncryptedUserEmail(string $email)
    {
        $user = User::findByEmail($email);
        if (!$user) {
            return null;
        }

        return DB::table('users')->where('id', $user->id)->first()->email;
    }

    public static function findByEmail(string $email): ?User
    {
        return User::whereBlind('email', 'email_index', $email)
            ->first();
    }

    public function getModelRoutesAttribute()
    {
        if (!auth()->user()) {
            return [
                'index' => null,
                'show' => null,
                'edit' => null,
                'destroy' => null,
            ];
        }

        $organization = Route::getCurrentRoute()->parameter('organization')
            ? Organization::find(Route::getCurrentRoute()->parameter('organization'))->first()
            : auth()->user()->currentOrganization;

        if ($this->hasRole('client')) {
            return [
                'index' => route('organizations.clients.index', [$organization]),
                'show' => $organization && $this->id ? route('organizations.clients.show', [$organization->id, $this->id]) : null,
                'edit' => $organization && $this->id ? route('organizations.clients.edit', [$organization->id, $this->id]) : null,
                'destroy' => $organization && $this->id ? route('organizations.clients.destroy', [$organization->id, $this->id]) : null,
            ];
        }

        if ($this->hasRole('counselor') || $this->hasRole('organization-admin') || $this->hasRole('super-admin')) {
            return [
                //'index'     => route('organizations.clients.index', [auth()->user()->currentOrganization->id]),
                'show' => $organization && $this->id ? route('organizations.employees.show', [$organization->id, $this->id]) : null,
                'edit' => $organization && $this->id ? route('organizations.employees.edit', [$organization->id, $this->id]) : null,
                'destroy' => $organization && $this->id ? route('organizations.employees.destroy', [$organization->id, $this->id]) : null,
            ];
        }

    }

    public function scopeIsOrganizationMember($query, Organization $organization)
    {
        return $query->whereHas('organizations', function ($query) use ($organization) {
            $query->whereIn('organization_id', [$organization->id]);
        });
    }

    public function scopeIsOrganization($query)
    {
        return $query->whereHas('roles', function ($query) {
            $query->where('name', 'organization');
        });
    }

    public function scopeIsClient($query)
    {
        return $query->whereHas('roles', function ($query) {
            $query->where('name', 'client');
        });
    }

    public function scopeIsActive($query)
    {
        return $query->where('is_active', true);
    }

    /* Relationships */

    public function scopeIsCounselorOrAdmin(Builder $query): void
    {
        $query->whereHas('roles', function ($query) {
            $query->where('name', 'counselor')
                ->orWhere('name', 'super-admin')
                ->orWhere('name', 'organization-admin');
        });
    }

    public function scopeCounselorClients(Builder $query): void
    {
        $query->where('counselor_id', auth()->user()->id);
    }

    public function currentOrganization(): BelongsTo
    {
        return $this->belongsTo(Organization::class, 'current_organization_id', 'id');
    }

    public function conversations(): BelongsToMany
    {
        return $this->belongsToMany(Conversation::class);
    }

    public function appointments(): BelongsToMany
    {
        return $this->belongsToMany(Appointment::class);
    }

    public function counselor(): BelongsTo
    {
        return $this->belongsTo(User::class, 'counselor_id', 'id');
    }

    /* Attributes */

    public function clients(): HasMany
    {
        return $this->hasMany(User::class, 'counselor_id', 'id');
    }

    public function scopeClientBelongsToUser($query)
    {
        return $query->whereHas('clients', function ($query) {
            $query->whereIn('counselor_id', [auth()->user()->id]);
        });
    }

    public function getFullNameAttribute()
    {
        return "{$this->first_name} {$this->last_name}";
    }

    public function getOrganizationIdAttribute()
    {
        if (session('organization_id')) {
            return session('organization_id');
        }

        $organization = $this->organizations()->first();
        if ($organization) {
            session(['organization_id' => $organization->id, 'organization_name' => $organization->name]);
            return $organization->id;
        }

        return NULL;
    }

    public function organizations(): BelongsToMany
    {
        return $this->belongsToMany(Organization::class);
    }

    public function user_meta(): HasOne
    {
        return $this->hasOne(UserMeta::class);
    }

    public function getOrganizationNameAttribute()
    {
        if (session('organization_id')) {
            return session('organization_id');
        }

        $organization = $this->organizations()->first();
        if ($organization) {
            session(['organization_id' => $organization->id, 'organization_name' => $organization->name]);
            return $organization->id;
        }

        return NULL;
    }

    public function conversation_notes(): HasMany
    {
        return $this->hasMany(ConversationNote::class);
    }


}
